@@ -24,9 +24,9 @@ namespace Sass {
     Listize(Memory_Manager&);
     ~Listize() { }
 
-    Expression* operator()(Selector_List*);
-    Expression* operator()(Complex_Selector*);
-    Expression* operator()(Compound_Selector*);
+    Expression* operator()(CommaSequence_Selector*);
+    Expression* operator()(Sequence_Selector*);
+    Expression* operator()(SimpleSequence_Selector*);
 
     template <typename U>
     Expression* fallback(U x) { return fallback_impl(x); }
