@@ -162,6 +162,14 @@ namespace Sass {
       return src;
     }
 
+    // equivalent of STRING_REGULAR_EXPRESSIONS
+    const char* re_string_double_open(const char* src);
+    const char* re_string_double_close(const char* src);
+    const char* re_string_single_open(const char* src);
+    const char* re_string_single_close(const char* src);
+    const char* re_string_uri_open(const char* src);
+    const char* re_string_uri_close(const char* src);
+
     // Match a line comment.
     const char* line_comment(const char* src);
 
@@ -216,6 +224,11 @@ namespace Sass {
     // const char* url_schema(const char* src);
     // const char* url_value(const char* src);
     const char* vendor_prefix(const char* src);
+
+    const char* re_special_directive(const char* src);
+    const char* re_prefixed_directive(const char* src);
+    const char* re_almost_any_value_token(const char* src);
+
     // Match CSS '@' keywords.
     const char* at_keyword(const char* src);
     const char* kwd_import(const char* src);
@@ -261,6 +274,7 @@ namespace Sass {
 
     const char* kwd_null(const char* src);
 
+    const char* re_selector_list(const char* src);
     const char* re_type_selector(const char* src);
     const char* re_static_expression(const char* src);
 
@@ -331,6 +345,7 @@ namespace Sass {
 
     // Match SCSS variable names.
     const char* variable(const char* src);
+    const char* calc_fn_call(const char* src);
 
     // IE stuff
     const char* ie_progid(const char* src);
