@@ -7,7 +7,8 @@
 
 namespace Sass {
 
-  double round(double x);
+
+
 
   /*
    This is for ports of functions in the Sass:Util module.
@@ -220,28 +221,28 @@ namespace Sass {
 
     std::map<unsigned int, KeyType> order;
 
-    std::map<KeyType, std::vector<EnumType> > grouped;
+    std::map<size_t, std::vector<EnumType> > grouped;
 
     for (typename std::vector<EnumType>::iterator enumIter = enumeration.begin(), enumIterEnd = enumeration.end(); enumIter != enumIterEnd; enumIter++) {
       EnumType& e = *enumIter;
 
       KeyType key = keyFunc(e);
 
-      if (grouped.find(key) == grouped.end()) {
+      if (grouped.find(key.hash()) == grouped.end()) {
         order.insert(std::make_pair((unsigned int)order.size(), key));
 
         std::vector<EnumType> newCollection;
         newCollection.push_back(e);
-        grouped.insert(std::make_pair(key, newCollection));
+        grouped.insert(std::make_pair(key.hash(), newCollection));
       } else {
-        std::vector<EnumType>& collection = grouped.at(key);
+        std::vector<EnumType>& collection = grouped.at(key.hash());
         collection.push_back(e);
       }
     }
 
     for (unsigned int index = 0; index < order.size(); index++) {
       KeyType& key = order.at(index);
-      std::vector<EnumType>& values = grouped.at(key);
+      std::vector<EnumType>& values = grouped.at(key.hash());
 
       std::pair<KeyType, std::vector<EnumType> > grouping = std::make_pair(key, values);
 
